@@ -1,9 +1,10 @@
 ;;; tpu-extras.el --- Scroll margins and free cursor mode for TPU-edt
+
 ;; Copyright (C) 1993 Free Software Foundation, Inc.
 
 ;; Author: Rob Riepel <riepel@networking.stanford.edu>
 ;; Maintainer: Rob Riepel <riepel@networking.stanford.edu>
-;; Keywords: tpu-edt
+;; Keywords: emulations
 
 ;; This file is part of GNU Emacs.
 
@@ -21,8 +22,6 @@
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-;;; Revision: $Id: tpu-extras.el,v 1.3 1993/08/09 06:18:02 rms Exp $
-
 ;;; Commentary:
 
 ;;  The functions contained in this file implement scroll margins and free
@@ -105,7 +104,7 @@
 
 ;;;  Revision Information
 
-(defconst tpu-extras-revision "$Revision: 1.3 $"
+(defconst tpu-extras-revision "$Revision: 1.6 $"
   "Revision number of the TPU-edt extras.")
 
 
@@ -133,7 +132,11 @@ the previous line when starting from a line beginning.")
 ;;;  Hooks  --  Set cursor free in picture mode.
 ;;;             Clean up when writing a file from cursor free mode.
 
-(add-hook 'edit-picture-hook 'tpu-set-cursor-free)
+(if tpu-gnu-emacs18-p
+    (or (memq 'tpu-set-cursor-free edit-picture-hook)
+	(setq edit-picture-hook
+	      (cons 'tpu-set-cursor-free edit-picture-hook)))
+  (add-hook 'picture-mode-hook 'tpu-set-cursor-free))
 
 (defun tpu-write-file-hook nil
   "Eliminate whitespace at ends of lines, if the cursor is free."
@@ -374,7 +377,7 @@ A repeat count means scroll that many sections."
 	 (far (save-excursion
 		(goto-char bottom) (forward-line (- height 2)) (point))))
     (tpu-search-internal-core pat quiet)
-    (if searching-forward
+    (if tpu-searching-forward
 	(cond((> (point) far)
 	      (setq left (save-excursion (forward-line height)))
 	      (if (= 0 left) (recenter top-margin)
@@ -425,6 +428,7 @@ version that respects the bottom scroll margin."
 
 ;;;  Function to set scroll margins
 
+;;;###autoload
 (defun tpu-set-scroll-margins (top bottom)
   "Set scroll margins."
   (interactive
@@ -449,12 +453,10 @@ version that respects the bottom scroll margin."
        (message "Scroll margins set.  Top = %s%%, Bottom = %s%%"
 		tpu-top-scroll-margin tpu-bottom-scroll-margin)))
 
-(fset 'set\ scroll\ margins 'tpu-set-scroll-margins)
-(fset 'SET\ SCROLL\ MARGINS 'tpu-set-scroll-margins)
-
 
 ;;;  Functions to set cursor bound or free
 
+;;;###autoload
 (defun tpu-set-cursor-free nil
   "Allow the cursor to move freely about the screen."
   (interactive)
@@ -464,6 +466,7 @@ version that respects the bottom scroll margin."
 			     GOLD-map)
   (message "The cursor will now move freely about the screen."))
 
+;;;###autoload
 (defun tpu-set-cursor-bound nil
   "Constrain the cursor to the flow of the text."
   (interactive)
@@ -474,15 +477,4 @@ version that respects the bottom scroll margin."
 			     GOLD-map)
   (message "The cursor is now bound to the flow of your text."))
 
-(fset 'set\ cursor\ bound 'tpu-set-cursor-bound)
-(fset 'SET\ CURSOR\ BOUND 'tpu-set-cursor-bound)
-(fset 'set\ cursor\ free 'tpu-set-cursor-free)
-(fset 'SET\ CURSOR\ FREE 'tpu-set-cursor-free)
-
-
-;;;  Keypad Mapping
-
-(define-key GOLD-map "F" 'tpu-set-cursor-free)			; F
-(define-key GOLD-map "f" 'tpu-set-cursor-free)			; f
-
 ;;; tpu-extras.el ends here
