@@ -52,7 +52,7 @@ static XMWinQue _XMWinQue;
  */
 int _XMErrorCode = XME_NO_ERROR; 
 /*
- * _XMErrorList - Global XMenu error code discription strings.
+ * _XMErrorList - Global XMenu error code description strings.
  */
 char *
 _XMErrorList[XME_CODE_COUNT] = {
@@ -95,7 +95,15 @@ _XMWinQueInit()
 	/*
 	 * Blank the queue structure.
 	 */
-	bzero(&_XMWinQue, sizeof(XMWinQue));
+	register int i;
+
+	for (i = 0; i < S_QUE_SIZE; i++)
+	  _XMWinQue.sq[i] = 0;
+
+	for (i = 0; i < P_QUE_SIZE; i++)
+	  _XMWinQue.pq[i] = 0;
+
+	_XMWinQue.sq_size = _XMWinQue.pq_size = 0;
 
 	/*
 	 * Initialize the next free location pointers.
@@ -288,7 +296,7 @@ _XMWinQueFlush(display, menu, pane, select)
 	    /*
 	     * Insert the new window id and its
 	     * associated XMSelect structure into the 
-	     * assoction table.
+	     * association table.
 	     */
 	    XMakeAssoc(display, menu->assoc_tab, s_ptr->window, s_ptr);
 	    XSelectInput(display, s_ptr->window, menu->s_events);
@@ -402,7 +410,7 @@ _XMRecomputeGlobals(display, menu)
     register int s_count = 0;		/* Maximum selection count. */
 
     int p_s_pad;		/* Pane <-> selection padding. */
-    int p_s_diff;		/* Pane <-> selection seperation. */
+    int p_s_diff;		/* Pane <-> selection separation. */
 
     int p_height;		/* Pane window height. */
     int p_width;		/* Pane window width. */
@@ -758,7 +766,7 @@ _XMRecomputeSelection(display, menu, s_ptr, s_num)
 	    s_ptr->label_x = s_ptr->window_x + ((menu->s_width - s_ptr->label_width) >> 1);
 	    break;
 	default:
-	    /* Error! Invaild style parameter. */
+	    /* Error! Invalid style parameter. */
 	    _XMErrorCode = XME_STYLE_PARAM;
 	    return(_FAILURE);
     }
@@ -781,7 +789,7 @@ _XMRecomputeSelection(display, menu, s_ptr, s_num)
  *		      the center of the current pane and selection to the 
  *		      the menu origin.
  *
- *	WARNING! ******	Be certain that all menu depencies have been
+ *	WARNING! ******	Be certain that all menu dependencies have been
  *			recomputed before calling this routine or
  *			unpredictable results will follow.
  */
