@@ -1,5 +1,5 @@
 /* systime.h - System-dependent definitions for time manipulations.
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -28,6 +28,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 #endif
 
+#ifdef HAVE_TZNAME
+#ifndef tzname		/* For SGI.  */
+extern char *tzname[];	/* RS6000 and others want it this way.  */
+#endif
+#endif
+
 /* SVr4 doesn't actually declare this in its #include files.  */
 #ifdef USG5_4
 extern long timezone;
@@ -39,6 +45,15 @@ extern long timezone;
 #endif
 #endif
 
+/* On some configurations (hpux8.0, X11R4), sys/time.h and X11/Xos.h
+   disagree about the name of the guard symbol.  */
+#ifdef HPUX
+#ifdef _STRUCT_TIMEVAL
+#ifndef __TIMEVAL__
+#define __TIMEVAL__
+#endif
+#endif
+#endif
 
 /* EMACS_TIME is the type to use to represent temporal intervals -
    struct timeval on some systems, int on others.  It can be passed as
@@ -80,11 +95,19 @@ extern long timezone;
 #define EMACS_SET_SECS(time, seconds)	    ((time).tv_sec  = (seconds))
 #define EMACS_SET_USECS(time, microseconds) ((time).tv_usec = (microseconds))
 
+/* On SVR4, the compiler may complain if given this extra BSD arg.  */
+#ifdef USG5_4
+#define EMACS_GET_TIME(time)                                  \
+{                                                             \
+  gettimeofday (&(time));                                     \
+}
+#else /* not USG5_4 */
 #define EMACS_GET_TIME(time)					\
 {								\
   struct timezone dummy;					\
   gettimeofday (&(time), &dummy);				\
 }
+#endif /* not USG5_4 */
 
 #define EMACS_ADD_TIME(dest, src1, src2)			\
 {								\
@@ -103,9 +126,9 @@ extern long timezone;
 }
 
 #define EMACS_TIME_NEG_P(time)					\
-  ((time).tv_sec < 0						\
+  ((long)(time).tv_sec < 0					\
    || ((time).tv_sec == 0					\
-       && (time).tv_usec < 0))
+       && (long)(time).tv_usec < 0))
 
 #else /* ! defined (HAVE_TIMEVAL) */
 
