@@ -1,5 +1,5 @@
 /* Define wait system call interface for Emacs.
-   Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,8 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Define the structure that the wait system call stores.
    On many systems, there is a structure defined for this.
@@ -24,17 +23,7 @@ Boston, MA 02111-1307, USA.  */
 
 #ifndef VMS
 #ifndef WAITTYPE
-
-#ifdef WAIT_USE_INT
-/* Some systems have  union wait  in their header, but we should use
-   int regardless of that.  */
-#include <sys/wait.h>
-#define WAITTYPE int
-#define WRETCODE(w) WEXITSTATUS (w)
-
-#else /* not WAIT_USE_INT */
-
-#if (!defined (BSD) && !defined (UNIPLUS) && !defined (STRIDE) && !(defined (HPUX) && !defined (NOMULTIPLEJOBS)) && !defined (HAVE_WAIT_HEADER))
+#if (!defined (BSD) && !defined (UNIPLUS) && !defined (STRIDE) && !(defined (HPUX) && !defined (NOMULTIPLEJOBS)) && !defined (HAVE_WAIT_HEADER)) || defined (LINUX)
 #define WAITTYPE int
 #define WIFSTOPPED(w) ((w&0377) == 0177)
 #define WIFSIGNALED(w) ((w&0377) != 0177 && (w&~0377) == 0)
@@ -45,9 +34,7 @@ Boston, MA 02111-1307, USA.  */
 #ifndef WCOREDUMP
 #define WCOREDUMP(w) ((w&0200) != 0)
 #endif
-
 #else 
-
 #ifdef BSD4_1
 #include <wait.h>
 #else
@@ -85,11 +72,8 @@ Boston, MA 02111-1307, USA.  */
 #define WIFEXITED(w) (WTERMSIG (w) == 0)
 #endif
 #endif /* BSD or UNIPLUS or STRIDE */
-#endif /* not WAIT_USE_INT */
 #endif /* no WAITTYPE */
-
 #else /* VMS */
-
 #define WAITTYPE int
 #define WIFSTOPPED(w) 0
 #define WIFSIGNALED(w) 0
@@ -102,5 +86,4 @@ Boston, MA 02111-1307, USA.  */
 #include <iodef.h>
 #include <clidef.h>
 #include "vmsproc.h"
-
 #endif /* VMS */
